﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.Xml;
using System.Threading;

namespace MPIAlgorithmRunner {
  class Program {
    EventWaitHandle waitHandle = new EventWaitHandle(false, EventResetMode.ManualReset);

    static void Main(string[] args) {
      if (args.Length != 2) {
        Console.WriteLine("Args:" + args.Length);

        throw new ArgumentException("You must provide two arguments - the algorithm file and the result file");
      }

      string fileName = args[0];
      string resultName = args[1];

      using (new MPI.Environment(ref args)) {
        Program p = new Program();
        p.StartAlgorithm(fileName, resultName + MPI.Communicator.world.Rank + ".hl");
      }
    }

    public void StartAlgorithm(string fileName, string resultName) {
      IAlgorithm alg = XmlParser.Deserialize<HeuristicLab.Optimization.IAlgorithm>(fileName);

      alg.Stopped += new EventHandler(algorithm_Stopped);
      waitHandle.Reset();

      alg.Prepare(true);
      alg.Start();

      waitHandle.WaitOne();

      XmlGenerator.Serialize(alg, resultName);
    }

    void algorithm_Stopped(object sender, EventArgs e) {
      waitHandle.Set();
    }
  }
}
